@@ -1,5 +1,7 @@
 <?php
 
+declare(strict_types=1);
+
 /*
  * This file is part of the TYPO3 CMS project.
  *
@@ -20,23 +22,21 @@ use TYPO3\CMS\Core\Utility\GeneralUtility;
 /**
  * Validator for email addresses
  */
-class EmailAddressValidator extends AbstractValidator
+final class EmailAddressValidator extends AbstractValidator
 {
+    protected string $message = 'LLL:EXT:extbase/Resources/Private/Language/locallang.xlf:validator.emailaddress.notvalid';
+
+    protected $supportedOptions = [
+        'message' => [null, 'Translation key or message for invalid value', 'string'],
+    ];
+
     /**
      * Checks if the given value is a valid email address.
-     *
-     * @param mixed $value The value that should be validated
      */
-    public function isValid($value)
+    public function isValid(mixed $value): void
     {
         if (!is_string($value) || !$this->validEmail($value)) {
-            $this->addError(
-                $this->translateErrorMessage(
-                    'validator.emailaddress.notvalid',
-                    'extbase'
-                ) ?? '',
-                1221559976
-            );
+            $this->addError($this->translateErrorMessage($this->message), 1221559976);
         }
     }
 
@@ -46,7 +46,7 @@ class EmailAddressValidator extends AbstractValidator
      * @param string $emailAddress Input string to evaluate
      * @return bool Returns TRUE if the $email address (input string) is valid
      */
-    protected function validEmail($emailAddress)
+    protected function validEmail(string $emailAddress): bool
     {
         return GeneralUtility::validEmail($emailAddress);
     }
