@@ -25,7 +25,7 @@ use TYPO3\CMS\Form\Mvc\Validation\Exception\InvalidValidationOptionsException;
  *
  * Scope: frontend
  */
-class DateRangeValidator extends AbstractValidator
+final class DateRangeValidator extends AbstractValidator
 {
     /**
      * @var array
@@ -39,9 +39,9 @@ class DateRangeValidator extends AbstractValidator
     /**
      * @param \DateTime $value The value that should be validated
      */
-    public function isValid($value)
+    public function isValid(mixed $value): void
     {
-        $this->validateOptions();
+        $options = $this->validateOptions();
 
         if (!($value instanceof \DateTime)) {
             $this->addError(
@@ -49,44 +49,37 @@ class DateRangeValidator extends AbstractValidator
                     'validation.error.1521293685',
                     'form',
                     [gettype($value)]
-                ) ?? '',
+                ),
                 1521293685
             );
-
             return;
         }
 
-        $minimum = $this->options['minimum'];
-        $maximum = $this->options['maximum'];
-        $format = $this->options['format'];
+        $minimum = $options['minimum'];
+        $maximum = $options['maximum'];
+        $format = $options['format'];
         $value->modify('midnight');
 
-        if (
-            $minimum instanceof \DateTime
-            && $value < $minimum
-        ) {
-            $this->addError(
-                $this->translateErrorMessage(
-                    'validation.error.1521293686',
-                    'form',
-                    [$minimum->format($format)]
-                ) ?? '',
-                1521293686,
-                [$minimum->format($format)]
-            );
-        }
-
-        if (
-            $maximum instanceof \DateTime
-            && $value > $maximum
-        ) {
+        if ($minimum instanceof \DateTime && $value < $minimum) {
             $this->addError(
                 $this->translateErrorMessage(
                     'validation.error.1521293687',
                     'form',
-                    [$maximum->format($format)]
-                ) ?? '',
+                    [$minimum->format($format)]
+                ),
                 1521293687,
+                [$minimum->format($format)]
+            );
+        }
+
+        if ($maximum instanceof \DateTime && $value > $maximum) {
+            $this->addError(
+                $this->translateErrorMessage(
+                    'validation.error.1521293686',
+                    'form',
+                    [$maximum->format($format)]
+                ),
+                1521293686,
                 [$maximum->format($format)]
             );
         }
@@ -97,17 +90,17 @@ class DateRangeValidator extends AbstractValidator
      *
      * @throws InvalidValidationOptionsException if the configured validation options are incorrect
      */
-    protected function validateOptions()
+    protected function validateOptions(): array
     {
+        $options = $this->options;
         if (!empty($this->options['minimum'])) {
             $minimum = \DateTime::createFromFormat($this->options['format'], $this->options['minimum']);
             if (!($minimum instanceof \DateTime)) {
                 $message = sprintf('The option "minimum" (%s) could not be converted to \DateTime from format "%s".', $this->options['minimum'], $this->options['format']);
                 throw new InvalidValidationOptionsException($message, 1521293813);
             }
-
             $minimum->modify('midnight');
-            $this->options['minimum'] = $minimum;
+            $options['minimum'] = $minimum;
         }
 
         if (!empty($this->options['maximum'])) {
@@ -116,9 +109,9 @@ class DateRangeValidator extends AbstractValidator
                 $message = sprintf('The option "maximum" (%s) could not be converted to \DateTime from format "%s".', $this->options['maximum'], $this->options['format']);
                 throw new InvalidValidationOptionsException($message, 1521293814);
             }
-
             $maximum->modify('midnight');
-            $this->options['maximum'] = $maximum;
+            $options['maximum'] = $maximum;
         }
+        return $options;
     }
 }
