@@ -19,40 +19,39 @@ namespace TYPO3\CMS\Redirects\Controller;
 
 use Psr\Http\Message\ResponseInterface;
 use Psr\Http\Message\ServerRequestInterface;
+use TYPO3\CMS\Backend\Attribute\AsController;
 use TYPO3\CMS\Backend\History\RecordHistory;
 use TYPO3\CMS\Backend\History\RecordHistoryRollback;
 use TYPO3\CMS\Core\DataHandling\Model\CorrelationId;
 use TYPO3\CMS\Core\Http\JsonResponse;
 use TYPO3\CMS\Core\Localization\LanguageService;
+use TYPO3\CMS\Core\Localization\LanguageServiceFactory;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
 use TYPO3\CMS\Redirects\Service\SlugService;
+use TYPO3\CMS\Redirects\Service\TemporaryPermissionMutationService;
 
 /**
  * @internal
  */
+#[AsController]
 class RecordHistoryRollbackController
 {
-    /**
-     * @var LanguageService
-     */
-    protected $languageService;
+    protected ?LanguageService $languageService = null;
 
-    public function __construct(LanguageService $languageService)
-    {
-        $this->languageService = $languageService;
-    }
+    public function __construct(
+        protected LanguageServiceFactory $languageServiceFactory,
+        protected RecordHistoryRollback $recordHistoryRollback,
+        protected TemporaryPermissionMutationService $temporaryPermissionMutationService
+    ) {}
 
-    /**
-     * @param ServerRequestInterface $request
-     * @return ResponseInterface
-     */
     public function revertCorrelation(ServerRequestInterface $request): ResponseInterface
     {
+        $this->languageService = $this->languageServiceFactory->createFromUserPreferences($GLOBALS['BE_USER']);
         $revertedCorrelationTypes = [];
         $correlationIds = $request->getQueryParams()['correlation_ids'] ?? [];
         /** @var CorrelationId[] $correlationIds */
         $correlationIds = array_map(
-            function (string $correlationId) {
+            static function (string $correlationId) {
                 return CorrelationId::fromString($correlationId);
             },
             $correlationIds
@@ -68,33 +67,45 @@ class RecordHistoryRollbackController
         $result = [
             'status' => 'error',
             'title' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:redirects_error_title'),
-            'message' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:redirects_error_message')
+            'message' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:redirects_error_message'),
         ];
         if (in_array('redirect', $revertedCorrelationTypes, true)) {
             $result = [
                 'status' => 'ok',
                 'title' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:revert_redirects_success_title'),
-                'message' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:revert_redirects_success_message')
+                'message' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:revert_redirects_success_message'),
             ];
             if (in_array('slug', $revertedCorrelationTypes, true)) {
                 $result = [
                     'status' => 'ok',
                     'title' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:revert_update_success_title'),
-                    'message' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:revert_update_success_message')
+                    'message' => $this->languageService->sL('LLL:EXT:redirects/Resources/Private/Language/locallang_slug_service.xlf:revert_update_success_message'),
                 ];
             }
         }
-        return (new JsonResponse())->setPayload($result);
+        return new JsonResponse($result);
     }
 
     protected function rollBackCorrelation(CorrelationId $correlationId): void
     {
-        $recordHistoryRollback = GeneralUtility::makeInstance(RecordHistoryRollback::class);
+        // Temporary add permissions to the user to perform the action.
+        // Store if we need to revert those changes after the actions.
+        $addedTableSelect = $this->temporaryPermissionMutationService->addTableSelect();
+        $addedTableModify = $this->temporaryPermissionMutationService->addTableModify();
+
         foreach (GeneralUtility::makeInstance(RecordHistory::class)->findEventsForCorrelation((string)$correlationId) as $recordHistoryEntry) {
             $element = $recordHistoryEntry['tablename'] . ':' . $recordHistoryEntry['recuid'];
             $tempRecordHistory = GeneralUtility::makeInstance(RecordHistory::class, $element);
             $tempRecordHistory->setLastHistoryEntryNumber((int)$recordHistoryEntry['uid']);
-            $recordHistoryRollback->performRollback('ALL', $tempRecordHistory->getDiff($tempRecordHistory->getChangeLog()));
+            $this->recordHistoryRollback->performRollback('ALL', $tempRecordHistory->getDiff($tempRecordHistory->getChangeLog()));
+        }
+
+        // Revert temporary permissions
+        if ($addedTableSelect) {
+            $this->temporaryPermissionMutationService->removeTableSelect();
+        }
+        if ($addedTableModify) {
+            $this->temporaryPermissionMutationService->removeTableModify();
         }
     }
 }
