@@ -1,20 +1,25 @@
 <?php
 
+use TYPO3\CMS\Redirects\Utility\RedirectConflict;
+
 return [
     'ctrl' => [
         'title' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect',
+        'descriptionColumn' => 'description',
         'label' => 'source_host',
         'label_alt' => 'source_path',
         'label_alt_force' => true,
         'crdate' => 'createdon',
-        'cruser_id' => 'createdby',
         'tstamp' => 'updatedon',
+        'hideTable' => true,
         'versioningWS' => false,
+        'groupName' => 'system',
         'default_sortby' => 'source_host, source_path',
-        'rootLevel' => 1,
+        'rootLevel' => -1,
         'security' => [
             'ignoreWebMountRestriction' => true,
             'ignoreRootLevelRestriction' => true,
+            'ignorePageTypeRestriction' => true,
         ],
         'delete' => 'deleted',
         'enablecolumns' => [
@@ -30,66 +35,33 @@ return [
     'types' => [
         '1' => [
             'showitem' => '
-                --div--;LLL:EXT:core/Resources/Private/Language/Form/locallang_tabs.xlf:general, --palette--;;source, --palette--;;targetdetails,
-                --div--;LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:tabs.redirectCount, disable_hitcount, hitcount, lasthiton,
-                --div--;LLL:EXT:core/Resources/Private/Language/Form/locallang_tabs.xlf:access, --palette--;;visibility'
+                --div--;LLL:EXT:core/Resources/Private/Language/Form/locallang_tabs.xlf:general, --palette--;;source, --palette--;;targetdetails, protected, --palette--;;internals,
+                --div--;LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:tabs.redirectCount, disable_hitcount, hitcount, lasthiton, createdon,
+                --div--;LLL:EXT:core/Resources/Private/Language/Form/locallang_tabs.xlf:access, --palette--;;visibility,
+                --div--;LLL:EXT:core/Resources/Private/Language/Form/locallang_tabs.xlf:notes, description',
         ],
     ],
     'palettes' => [
         'visibility' => [
-            'showitem' => 'disabled, --linebreak--, starttime, endtime'
+            'showitem' => 'disabled, --linebreak--, starttime, endtime',
         ],
         'source' => [
-            'showitem' => 'source_host, --linebreak--, source_path, respect_query_parameters, is_regexp'
+            'showitem' => 'source_host, --linebreak--, source_path, respect_query_parameters, is_regexp',
         ],
         'targetdetails' => [
-            'showitem' => 'target, target_statuscode, --linebreak--, force_https, keep_query_parameters'
+            'showitem' => 'target, target_statuscode, --linebreak--, force_https, keep_query_parameters',
+        ],
+        'internals' => [
+            'showitem' => 'creation_type, integrity_status',
         ],
     ],
     'columns' => [
-        'disabled' => [
-            'exclude' => true,
-            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.enabled',
-            'config' => [
-                'type' => 'check',
-                'renderType' => 'checkboxToggle',
-                'items' => [
-                    [
-                        0 => '',
-                        1 => '',
-                        'invertStateDisplay' => true
-                    ]
-                ],
-            ]
-        ],
-        'starttime' => [
-            'exclude' => true,
-            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.starttime',
-            'config' => [
-                'type' => 'input',
-                'renderType' => 'inputDateTime',
-                'eval' => 'datetime,int',
-                'default' => 0
-            ]
-        ],
-        'endtime' => [
-            'exclude' => true,
-            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.endtime',
-            'config' => [
-                'type' => 'input',
-                'renderType' => 'inputDateTime',
-                'eval' => 'datetime,int',
-                'default' => 0,
-                'range' => [
-                    'upper' => mktime(0, 0, 0, 1, 1, 2038)
-                ]
-            ]
-        ],
         'source_host' => [
             'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.source_host',
             'config' => [
                 'type' => 'input',
-                'eval' => 'trim,required,' . \TYPO3\CMS\Redirects\Evaluation\SourceHost::class,
+                'required' => true,
+                'eval' => 'trim,' . \TYPO3\CMS\Redirects\Evaluation\SourceHost::class,
                 // items will be extended by local sys_domain records using dataprovider TYPO3\CMS\Redirects\FormDataProvider\ValuePickerItemDataProvider
                 'valuePicker' => [
                     'items' => [
@@ -106,9 +78,10 @@ return [
             'config' => [
                 'type' => 'input',
                 'size' => 30,
-                'eval' => 'trim,required',
+                'required' => true,
+                'eval' => 'trim',
                 'placeholder' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_module_redirect.xlf:source_path.placeholder',
-                'max' => 2048
+                'max' => 2048,
             ],
         ],
         'force_https' => [
@@ -118,13 +91,7 @@ return [
                 'type' => 'check',
                 'renderType' => 'checkboxToggle',
                 'default' => 0,
-                'items' => [
-                    [
-                        0 => '',
-                        1 => '',
-                    ]
-                ],
-            ]
+            ],
         ],
         'keep_query_parameters' => [
             'exclude' => true,
@@ -133,13 +100,7 @@ return [
                 'type' => 'check',
                 'renderType' => 'checkboxToggle',
                 'default' => 0,
-                'items' => [
-                    [
-                        0 => '',
-                        1 => '',
-                    ]
-                ],
-            ]
+            ],
         ],
         'respect_query_parameters' => [
             'exclude' => true,
@@ -148,30 +109,17 @@ return [
                 'type' => 'check',
                 'renderType' => 'checkboxToggle',
                 'default' => 0,
-                'items' => [
-                    [
-                        0 => '',
-                        1 => '',
-                    ]
-                ],
-            ]
+            ],
         ],
         'target' => [
             'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target',
             'config' => [
-                'type' => 'input',
-                'eval' =>'required',
-                'renderType' => 'inputLink',
-                'softref' => 'typolink',
-                'fieldControl' => [
-                    'linkPopup' => [
-                        'options' => [
-                            'blindLinkOptions' => 'mail,folder,telephone',
-                            'blindLinkFields' => 'class, target, title'
-                        ],
-                    ],
+                'type' => 'link',
+                'required' => true,
+                'allowedTypes' => ['page', 'file', 'url', 'record'],
+                'appearance' => [
+                    'allowedOptions' => ['params', 'rel'],
                 ],
-                'max' => 2048
             ],
         ],
         'target_statuscode' => [
@@ -181,13 +129,37 @@ return [
                 'type' => 'select',
                 'renderType' => 'selectSingle',
                 'items' => [
-                    ['LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.301', 301],
-                    ['LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.302', 302],
-                    ['LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.303', 303],
-                    ['LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.307', 307],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.301',
+                        'value' => 301,
+                        'group' => 'change',
+                    ],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.302',
+                        'value' => 302,
+                        'group' => 'change',
+                    ],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.303',
+                        'value' => 303,
+                        'group' => 'change',
+                    ],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.307',
+                        'value' => 307,
+                        'group' => 'keep',
+                    ],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.308',
+                        'value' => 308,
+                        'group' => 'keep',
+                    ],
+                ],
+                'itemGroups' => [
+                    'keep' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.keep',
+                    'change' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.target_statuscode.change',
                 ],
                 'default' => 307,
-                'size' => 1,
             ],
         ],
         'hitcount' => [
@@ -197,20 +169,27 @@ return [
                 'type' => 'input',
                 'size' => 5,
                 'default' => 0,
-                'readOnly' => true
+                'readOnly' => true,
             ],
-            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled'
+            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled',
         ],
         'lasthiton' => [
             'exclude' => true,
             'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.lasthiton',
             'config' => [
-                'type' => 'input',
-                'eval' => 'datetime,int',
-                'renderType' => 'inputDateTime',
-                'readOnly' => true
+                'type' => 'datetime',
+                'readOnly' => true,
             ],
-            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled'
+            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled',
+        ],
+        'createdon' => [
+            'exclude' => true,
+            'label' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.creationDate',
+            'config' => [
+                'type' => 'datetime',
+                'readOnly' => true,
+            ],
+            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled',
         ],
         'disable_hitcount' => [
             'exclude' => true,
@@ -220,15 +199,14 @@ return [
                 'renderType' => 'checkboxLabeledToggle',
                 'items' => [
                     [
-                        0 => '',
-                        1 => '',
+                        'label' => '',
                         'labelChecked' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.enabled',
                         'labelUnchecked' => 'LLL:EXT:core/Resources/Private/Language/locallang_general.xlf:LGL.disabled',
-                        'invertStateDisplay' => true
-                    ]
+                        'invertStateDisplay' => true,
+                    ],
                 ],
             ],
-            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled'
+            'displayCond' => 'USER:TYPO3\CMS\Redirects\UserFunctions\HitCountDisplayCondition->isEnabled',
         ],
         'is_regexp' => [
             'exclude' => true,
@@ -236,13 +214,57 @@ return [
             'config' => [
                 'type' => 'check',
                 'renderType' => 'checkboxToggle',
+            ],
+        ],
+        'protected' => [
+            'exclude' => true,
+            'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.protected',
+            'description' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.protected.description',
+            'config' => [
+                'type' => 'check',
+                'renderType' => 'checkboxToggle',
+            ],
+        ],
+        'creation_type' => [
+            'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.creation_type',
+            'description' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.creation_type.description',
+            'config' => [
+                'type' => 'select',
+                'renderType' => 'selectSingle',
                 'items' => [
                     [
-                        0 => '',
-                        1 => '',
-                    ]
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.creation_type.0',
+                        'value' => 0,
+                    ],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.creation_type.1',
+                        'value' => 1,
+                    ],
                 ],
-            ]
+                'default' => 1,
+                'readOnly' => true,
+            ],
+        ],
+        'integrity_status' => [
+            'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.integrity_status',
+            'description' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.integrity_status.description',
+            'config' => [
+                'type' => 'select',
+                'renderType' => 'selectSingle',
+                'dbFieldLength' => 180,
+                'default' => RedirectConflict::NO_CONFLICT,
+                'items' => [
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.integrity_status.no_conflict',
+                        'value' => RedirectConflict::NO_CONFLICT,
+                    ],
+                    [
+                        'label' => 'LLL:EXT:redirects/Resources/Private/Language/locallang_db.xlf:sys_redirect.integrity_status.self_reference',
+                        'value' => RedirectConflict::SELF_REFERENCE,
+                    ],
+                ],
+                'readOnly' => true,
+            ],
         ],
     ],
 ];
