@@ -1,5 +1,6 @@
 <?php
-namespace TYPO3\CMS\Workspaces\Service\Dependency;
+
+declare(strict_types=1);
 
 /*
  * This file is part of the TYPO3 CMS project.
@@ -14,118 +15,66 @@ namespace TYPO3\CMS\Workspaces\Service\Dependency;
  * The TYPO3 project - inspiring people to share!
  */
 
+namespace TYPO3\CMS\Workspaces\Service\Dependency;
+
+use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
 use TYPO3\CMS\Core\SingletonInterface;
 use TYPO3\CMS\Core\Utility\GeneralUtility;
-use TYPO3\CMS\Workspaces\Dependency;
-use TYPO3\CMS\Workspaces\Service\GridDataService;
+use TYPO3\CMS\Workspaces\Dependency\DependencyResolver;
+use TYPO3\CMS\Workspaces\Dependency\ElementEntity;
+use TYPO3\CMS\Workspaces\Dependency\ElementEntityProcessor;
+use TYPO3\CMS\Workspaces\Dependency\EventCallback;
+use TYPO3\CMS\Workspaces\Dependency\ReferenceEntity;
 
 /**
  * Service to collect dependent elements.
+ *
+ * @internal
  */
 class CollectionService implements SingletonInterface
 {
-    /**
-     * @var \TYPO3\CMS\Core\DataHandling\DataHandler
-     */
-    protected $dataHandler;
+    protected ?ElementEntityProcessor $elementEntityProcessor;
+    protected ?DependencyResolver $dependencyResolver = null;
+    protected array $dataArray;
+    protected array $nestedDataArray;
 
-    /**
-     * @var Dependency\ElementEntityProcessor
-     */
-    protected $elementEntityProcessor;
-
-    /**
-     * @var Dependency\DependencyResolver
-     */
-    protected $dependencyResolver;
-
-    /**
-     * @var array
-     */
-    protected $dataArray;
-
-    /**
-     * @var array
-     */
-    protected $nestedDataArray;
-
-    /**
-     * @return Dependency\DependencyResolver
-     */
-    public function getDependencyResolver()
+    public function getDependencyResolver(): DependencyResolver
     {
         if (!isset($this->dependencyResolver)) {
-            $this->dependencyResolver = GeneralUtility::makeInstance(Dependency\DependencyResolver::class);
-            $this->dependencyResolver->setOuterMostParentsRequireReferences(true);
-            $this->dependencyResolver->setWorkspace($this->getWorkspace());
-
+            $this->dependencyResolver = GeneralUtility::makeInstance(DependencyResolver::class);
+            $this->dependencyResolver->setWorkspace($this->getBackendUser()->workspace);
             $this->dependencyResolver->setEventCallback(
-                Dependency\ElementEntity::EVENT_Construct,
-                $this->getDependencyCallback('createNewDependentElementCallback')
+                ElementEntity::EVENT_Construct,
+                GeneralUtility::makeInstance(EventCallback::class, $this->getElementEntityProcessor(), 'createNewDependentElementCallback', ['workspace' => $this->getBackendUser()->workspace])
             );
-
             $this->dependencyResolver->setEventCallback(
-                Dependency\ElementEntity::EVENT_CreateChildReference,
-                $this->getDependencyCallback('createNewDependentElementChildReferenceCallback')
+                ElementEntity::EVENT_CreateChildReference,
+                GeneralUtility::makeInstance(EventCallback::class, $this->getElementEntityProcessor(), 'createNewDependentElementChildReferenceCallback')
             );
-
             $this->dependencyResolver->setEventCallback(
-                Dependency\ElementEntity::EVENT_CreateParentReference,
-                $this->getDependencyCallback('createNewDependentElementParentReferenceCallback')
+                ElementEntity::EVENT_CreateParentReference,
+                GeneralUtility::makeInstance(EventCallback::class, $this->getElementEntityProcessor(), 'createNewDependentElementParentReferenceCallback')
             );
         }
-
         return $this->dependencyResolver;
     }
 
-    /**
-     * Gets a new callback to be used in the dependency resolver utility.
-     *
-     * @param string $method
-     * @param array $targetArguments
-     * @return Dependency\EventCallback
-     */
-    protected function getDependencyCallback($method, array $targetArguments = [])
-    {
-        return GeneralUtility::makeInstance(
-            Dependency\EventCallback::class,
-            $this->getElementEntityProcessor(),
-            $method,
-            $targetArguments
-        );
-    }
-
     /**
      * Gets the element entity processor.
-     *
-     * @return Dependency\ElementEntityProcessor
      */
-    protected function getElementEntityProcessor()
+    protected function getElementEntityProcessor(): ElementEntityProcessor
     {
         if (!isset($this->elementEntityProcessor)) {
-            $this->elementEntityProcessor = GeneralUtility::makeInstance(Dependency\ElementEntityProcessor::class);
-            $this->elementEntityProcessor->setWorkspace($this->getWorkspace());
+            $this->elementEntityProcessor = GeneralUtility::makeInstance(ElementEntityProcessor::class);
+            $this->elementEntityProcessor->setWorkspace($this->getBackendUser()->workspace);
         }
         return $this->elementEntityProcessor;
     }
 
-    /**
-     * Gets the current workspace id.
-     *
-     * @return int
-     */
-    protected function getWorkspace()
-    {
-        return (int)$GLOBALS['BE_USER']->workspace;
-    }
-
     /**
      * Processes the data array
-     *
-     * @param array $dataArray
-     * @return array
      */
-    public function process(array $dataArray)
+    public function process(array $dataArray): array
     {
         $collection = 0;
         $this->dataArray = $dataArray;
@@ -147,8 +96,8 @@ class CollectionService implements SingletonInterface
 
         $processedDataArray = $this->finalize($this->dataArray);
 
-        unset($this->dataArray);
-        unset($this->nestedDataArray);
+        $this->dataArray = [];
+        $this->nestedDataArray = [];
 
         return $processedDataArray;
     }
@@ -156,11 +105,8 @@ class CollectionService implements SingletonInterface
     /**
      * Applies structures to instance data array and
      * ensures children are added below accordant parent
-     *
-     * @param array $dataArray
-     * @return array
      */
-    protected function finalize(array $dataArray)
+    protected function finalize(array $dataArray): array
     {
         $processedDataArray = [];
         foreach ($dataArray as $dataElement) {
@@ -181,22 +127,17 @@ class CollectionService implements SingletonInterface
 
     /**
      * Resolves nested child dependencies.
-     *
-     * @param Dependency\ElementEntity $parent
-     * @param int $collection
-     * @param string $nextParentIdentifier
-     * @param int $collectionLevel
      */
-    protected function resolveDataArrayChildDependencies(Dependency\ElementEntity $parent, $collection, $nextParentIdentifier = '', $collectionLevel = 0)
+    protected function resolveDataArrayChildDependencies(ElementEntity $parent, int $collection, string $nextParentIdentifier = '', int $collectionLevel = 0): void
     {
         $parentIdentifier = $parent->__toString();
         $parentIsSet = isset($this->dataArray[$parentIdentifier]);
 
         if ($parentIsSet) {
-            $this->dataArray[$parentIdentifier][GridDataService::GridColumn_Collection] = $collection;
-            $this->dataArray[$parentIdentifier][GridDataService::GridColumn_CollectionLevel] = $collectionLevel;
-            $this->dataArray[$parentIdentifier][GridDataService::GridColumn_CollectionCurrent] = md5($parentIdentifier);
-            $this->dataArray[$parentIdentifier][GridDataService::GridColumn_CollectionChildren] = count($parent->getChildren());
+            $this->dataArray[$parentIdentifier]['Workspaces_Collection'] = $collection;
+            $this->dataArray[$parentIdentifier]['Workspaces_CollectionLevel'] = $collectionLevel;
+            $this->dataArray[$parentIdentifier]['Workspaces_CollectionCurrent'] = md5($parentIdentifier);
+            $this->dataArray[$parentIdentifier]['Workspaces_CollectionChildren'] = $this->getCollectionChildrenCount($parent->getChildren());
             $nextParentIdentifier = $parentIdentifier;
             $collectionLevel++;
         }
@@ -213,10 +154,29 @@ class CollectionService implements SingletonInterface
             if (!empty($nextParentIdentifier) && isset($this->dataArray[$childIdentifier])) {
                 // Remove from dataArray, but collect to process later
                 // and add it just next to the accordant parent element
-                $this->dataArray[$childIdentifier][GridDataService::GridColumn_CollectionParent] = md5($nextParentIdentifier);
+                $this->dataArray[$childIdentifier]['Workspaces_CollectionParent'] = md5($nextParentIdentifier);
                 $this->nestedDataArray[$nextParentIdentifier][] = $this->dataArray[$childIdentifier];
                 unset($this->dataArray[$childIdentifier]);
             }
         }
     }
+
+    /**
+     * Return count of children, present in the data array
+     *
+     * @param ReferenceEntity[] $children
+     */
+    protected function getCollectionChildrenCount(array $children): int
+    {
+        return count(
+            array_filter($children, function (ReferenceEntity $child) {
+                return isset($this->dataArray[$child->getElement()->__toString()]);
+            })
+        );
+    }
+
+    protected function getBackendUser(): BackendUserAuthentication
+    {
+        return $GLOBALS['BE_USER'];
+    }
 }
